/**
 * Copyright 2011-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.deploymentplan;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugins.annotations.Component;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.apache.maven.plugins.annotations.ResolutionScope;
import org.apache.maven.project.DefaultDependencyResolutionRequest;
import org.apache.maven.project.DependencyResolutionException;
import org.apache.maven.project.DependencyResolutionRequest;
import org.apache.maven.project.DependencyResolutionResult;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.ProjectDependenciesResolver;
import org.codehaus.plexus.util.FileUtils;
import org.ow2.util.maven.deploymentplan.filter.DirectDependencyFilter;
import org.ow2.util.maven.deploymentplan.filter.PlanDependencyFilter;
import org.ow2.util.maven.deploymentplan.util.Artifacts;
import org.sonatype.aether.RepositorySystemSession;
import org.sonatype.aether.artifact.Artifact;
import org.sonatype.aether.graph.Dependency;
import org.sonatype.aether.graph.DependencyFilter;
import org.sonatype.aether.util.filter.AndDependencyFilter;

/**
 * A {@code FlatDirectoryProvisioningMojo} is responsible to build a flat directory
 * structure from deployment-plans defined in this project's dependency tree
 * (only plans declared on the first level: direct dependencies).
 *
 * @author Guillaume Sauthier
 * @since 2.0.0
 */
@Mojo(name = "provision-directory",
      defaultPhase = LifecyclePhase.GENERATE_RESOURCES,
      requiresDependencyResolution = ResolutionScope.COMPILE)
public class FlatDirectoryProvisioningMojo extends AbstractMojo {

    @Component
    private MavenProject project;

    @Parameter(defaultValue = "${repositorySystemSession}", readonly = true)   // @Component doesn't work !?
    private RepositorySystemSession repositorySession;

    @Component
    private ProjectDependenciesResolver resolver;

    /**
     * The directory where plans will be placed.
     * @since 2.0.0
     */
    @Parameter(alias = "repositoryDirectory",
               defaultValue = "${project.build.directory}/flat-repository")
    private File repository;

    /**
     * Map artifact's name onto a new name, see {@link Enrichment}.
     * @since 2.0.0
     */
    @Parameter
    private List<Enrichment> enrichments = new ArrayList<Enrichment>();

    public void execute() throws MojoExecutionException, MojoFailureException {

        try {

            // Resolve all dependencies
            DependencyResolutionRequest request = new DefaultDependencyResolutionRequest(project, repositorySession);
            request.setResolutionFilter(planDirectDependenciesFilter());
            DependencyResolutionResult result = resolver.resolve(request);

            // Iterates on the collected nodes
            List<Dependency> nodes = result.getResolvedDependencies();
            for (Dependency node : nodes) {

                // Ensure the artifact is resolved
                Artifact artifact = node.getArtifact();
                getLog().debug("Selected artifact: " + artifact);

                // Copy the artifact into the defined destination
                String name = getMappedName(Artifacts.getDependencyConflictId(artifact));
                if (name == null) {
                    // No mapping defined
                    name = artifact.getArtifactId();
                }
                File destination = new File(repository, name  + ".xml");

                getLog().debug("Copy artifact: " + artifact.getFile() + " to " + destination);
                try {
                    FileUtils.copyFileIfModified(artifact.getFile(), destination);
                } catch (IOException e) {
                    throw new MojoFailureException(e.getMessage());
                }

            }
        } catch (DependencyResolutionException e) {
            throw new MojoFailureException(e.getMessage());
        } catch (NullPointerException npe) {
            throw new MojoFailureException(npe.getMessage());
        }
    }

    private DependencyFilter planDirectDependenciesFilter() {
        return new AndDependencyFilter(
                new PlanDependencyFilter(),
                new DirectDependencyFilter());
    }

    private String getMappedName(String id) {
        for (Enrichment enrichment : enrichments) {
            if (id.equals(enrichment.getArtifact())) {
                return enrichment.getMappedName();
            }
        }
        return null;
    }
}
