/**
 * Copyright 2011-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.deploymentplan;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugins.annotations.Component;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.apache.maven.plugins.annotations.ResolutionScope;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.MavenProjectHelper;
import org.codehaus.plexus.util.IOUtil;

/**
 * A {@code GeneratePlanMojo} is responsible to build a deployment plan XML
 * file using the information available in the POM.
 *
 * @author Guillaume Sauthier
 * @since 2.0.0
 */
@Mojo(name = "generate",
      defaultPhase = LifecyclePhase.GENERATE_RESOURCES,
      requiresDependencyResolution = ResolutionScope.COMPILE)
public class GeneratePlanMojo extends AbstractMojo {

    @Component
    private MavenProject project;

    @Component
    private MavenProjectHelper projectHelper;

    /**
     * The XML file that will contain the geenrated plan.
     * @since 2.0.0
     */
    @Parameter(alias = "targetPlanFile",
               defaultValue = "${project.build.directory}/${project.artifactId}-${project.version}.xml")
    private File target;

    /**
     * List of reloadables artifacts in condensed form ({@literal groupId:artifactId:type:classifier}).
     * @since 2.0.0
     */
    @Parameter
    private List<String> reloadables = new ArrayList<String>();

    /**
     * List of artifacts in condensed form ({@literal groupId:artifactId:type:classifier}) not to be started.
     * @since 2.0.0
     */
    @Parameter(alias = "not-started")
    private List<String> notStarted = new ArrayList<String>();

    /**
     * List of artifacts in condensed form ({@literal groupId:artifactId:type:classifier}) to
     * be declared as {@literal reference:} bundles.
     * @since 2.0.0
     */
    @Parameter
    private List<String> references = new ArrayList<String>();

    /**
     * Express startlevels of artifacts in condensed form ({@literal groupId:artifactId:type:classifier}).
     * <pre>{@code <startlevel>
     *     <level-1>
     *         com.acme:shop:jar,
     *         com.acme.foo:application:jar
     *     </level-1>
     *     <level-4>
     *         com.acme.bar:business:jar
     *     </level-4>
     * </startlevel>}</pre>
     * @since 2.0.0
     */
    @Parameter
    private Map<String, String> startlevel = new HashMap<String, String>();

    /**
     * List of transient artifacts in condensed form ({@literal groupId:artifactId:type:classifier}).
     * @since 2.0.0
     */
    @Parameter
    private List<String> transients = new ArrayList<String>();

    /**
     * Is the generated plan atomic ?
     * @since 2.0.0
     */
    @Parameter(defaultValue = "false")
    private boolean atomic = false;

    /**
     * Enrich artifacts description with per-artifact information, see {@link Enrichment}.
     * @since 2.0.0
     */
    @Parameter
    private List<Enrichment> enrichments = new ArrayList<Enrichment>();

    /**
     * The condition that have to be satisfied for this plan to be deployed at runtime.
     * @since 2.0.0
     */
    @Parameter(alias = "global-condition")
    private String globalCondition;

    /**
     * Allow timestamp in SNAPSHOT version if set to true
     * @since 2.0.0
     */
    @Parameter(defaultValue = "false")
    private boolean allowTimestamp = false;

    public void execute() throws MojoExecutionException, MojoFailureException {

        // Build up the plan using this project's dependencies
        // The main advantage is that theses resources have already be resolved by maven

        FileOutputStream fos = null;
        try {

            // Prepare the target directory
            File parent = target.getParentFile();
            if (!parent.exists()) {
                parent.mkdirs();
            }

            // Get our OutputStream
            fos = new FileOutputStream(target);
            PrintStream stream = new PrintStream(fos);

            // Dump the plan
            writeDeploymentPlan(stream);

            // Attach the generated plan
            project.getArtifact().setFile(target);

        } catch (IOException ioe) {
            getLog().error(ioe);
            throw new MojoFailureException(ioe.getMessage());
        } finally {
            // Close the stream
            IOUtil.close(fos);
        }
    }

    @SuppressWarnings("unchecked")
    private void writeDeploymentPlan(PrintStream stream) {
        writeHeader(stream, getPlanName());

        Set<Artifact> artifacts = project.getDependencyArtifacts();

        for (Artifact artifact : artifacts) {
             // Ignore provided scope
             if (!"provided".equals(artifact.getScope())) {
                 writeMavenDeployment(stream, artifact);
             }
        }

        writeFooter(stream);
    }

    private String getPlanName() {
        return project.getGroupId() + ":" + project.getArtifactId() + ":" + project.getVersion();
    }

    private void writeHeader(PrintStream stream, String id) {

        stream.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        stream.println("<!-- Generated the " + new Date() + ". -->");
        stream.println("<deployment-plan xmlns=\"http://jonas.ow2.org/ns/deployment-plan/1.0\"");
        stream.println("                 xmlns:m2=\"http://jonas.ow2.org/ns/deployment-plan/maven2/1.0\"");
        stream.println("                 xmlns:url=\"http://jonas.ow2.org/ns/deployment-plan/url/1.0\"");
        stream.println("                 xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");
        stream.println("                 xsi:schemaLocation=\"http://jonas.ow2.org/ns/deployment-plan/1.0");
        stream.println("                                     http://jonas.ow2.org/ns/deployment-plan/1.0/deployment-plan-1.0.1.xsd");
        stream.println("                                     http://jonas.ow2.org/ns/deployment-plan/maven2/1.0");
        stream.println("                                     http://jonas.ow2.org/ns/deployment-plan/maven2-deployment-plan-1.0.xsd\"");
        if (globalCondition != null) {
            stream.println("                 condition=\"" + globalCondition + "\"");
        }
        stream.println("                 atomic=\"" + atomic + "\" id=\"" + id + "\">");

    }

    private void writeMavenDeployment(PrintStream stream, Artifact artifact) {

        String id = artifact.getDependencyConflictId();
        stream.println("  <deployment id=\"" + id + "\"");
        stream.print("              xsi:type=\"m2:maven2-deploymentType\"");
        if (isReloadable(id)) {
            stream.println();
            stream.print("              reloadable=\"true\"");
        }
        if (isNotStarted(id)) {
            stream.println();
            stream.print("              start=\"false\"");
        }
        if (isReference(id)) {
            stream.println();
            stream.print("              reference=\"true\"");
        }
        if (hasStartLevel(id)) {
            stream.println();
            stream.print("              startlevel=\"" + getStartLevel(id) + "\"");
        }
        if (isStartedTransientely(id)) {
            stream.println();
            stream.print("              starttransient=\"true\"");
        }
        if (isConditional(id)) {
            stream.println();
            stream.print("              condition=\"" + getCondition(id) + "\"");
        }
        stream.println(">");

        stream.println("    <m2:groupId>" + artifact.getGroupId() + "</m2:groupId>");
        stream.println("    <m2:artifactId>" + artifact.getArtifactId() + "</m2:artifactId>");
        if (!isNullOrEmpty(artifact.getClassifier())) {
            stream.println("    <m2:classifier>" + artifact.getClassifier() + "</m2:classifier>");
        }

        // Get the real artifact version (possibly with a timestamp)
        String version = artifact.getVersion();
        if (!allowTimestamp) {
            // Force to use base version (no timestamp)
            version = artifact.getBaseVersion();
        }
        stream.println("    <m2:version>" + version + "</m2:version>");
        stream.println("    <m2:type>" + artifact.getType() + "</m2:type>");

        stream.println("  </deployment>");

    }

    private boolean isNullOrEmpty(String value) {
        return (value == null) || ("".equals(value));
    }

    private String getCondition(String id) {
        for (Enrichment enrichment : enrichments) {
            if (id.equals(enrichment.getArtifact())) {
                return enrichment.getCondition();
            }
        }
        return null;
    }

    private boolean isConditional(String id) {
        for (Enrichment enrichment : enrichments) {
            if (id.equals(enrichment.getArtifact())) {
                return true;
            }
        }
        return false;
    }

    private boolean isStartedTransientely(String id) {
        return transients.contains(id);
    }

    private String getStartLevel(String id) {
        for (Map.Entry<String, String> entry : startlevel.entrySet()) {
            if (entry.getValue().contains(id)) {
                String element = entry.getKey();
                return element.substring(element.lastIndexOf("-") + 1);
            }
        }
        return "1";
    }

    private boolean hasStartLevel(String id) {
        for (Map.Entry<String, String> entry : startlevel.entrySet()) {
            if (entry.getValue().contains(id)) {
                return true;
            }
        }
        return false;
    }

    private boolean isReference(String id) {
        return references.contains(id);
    }

    private boolean isNotStarted(String id) {
        return notStarted.contains(id);
    }

    private boolean isReloadable(String id) {
        return reloadables.contains(id);
    }

    private void writeFooter(PrintStream stream) {
        stream.println("</deployment-plan>");
    }

    public void setTarget(File target) {
        this.target = target;
    }

    /**
     * @return true if timestamp are allowed in SNAPSHOT version
     */
    public boolean isAllowTimestamp() {
        return this.allowTimestamp;
    }

    /**
     * @param allowTimestamp Allow timestamp in SNAPSHOT version if set to true
     */
    public void setAllowTimestamp(final boolean allowTimestamp) {
        this.allowTimestamp = allowTimestamp;
    }
}
