/**
 * Copyright 2011-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.deploymentplan;

import java.io.File;
import java.io.IOException;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.repository.DefaultRepositoryRequest;
import org.apache.maven.artifact.repository.layout.ArtifactRepositoryLayout;
import org.apache.maven.execution.MavenSession;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugins.annotations.Component;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.apache.maven.plugins.annotations.ResolutionScope;
import org.apache.maven.project.MavenProject;
import org.apache.maven.repository.RepositorySystem;
import org.apache.maven.shared.dependency.graph.DependencyGraphBuilder;
import org.apache.maven.shared.dependency.graph.DependencyGraphBuilderException;
import org.apache.maven.shared.dependency.graph.DependencyNode;
import org.codehaus.plexus.util.FileUtils;
import org.ow2.util.maven.deploymentplan.visitor.PlanAndChildrenCollector;

/**
 * A {@code ProvisionRepositoryMojo} is responsible to build a maven-like directory
 * structure from deployment-plans defined in this project's dependency tree.
 *
 * @author Guillaume Sauthier
 * @since 2.0.0
 */
@Mojo(name = "provision-repository",
      defaultPhase = LifecyclePhase.GENERATE_RESOURCES,
      requiresDependencyResolution = ResolutionScope.COMPILE)
public class RepositoryProvisioningMojo extends AbstractMojo {

    @Component
    private MavenProject project;

    @Component
    private MavenSession mavenSession;

    @Component
    private RepositorySystem repositorySystem;

    @Component(hint = "default")
    private ArtifactRepositoryLayout layout;

    @Component
    private DependencyGraphBuilder graphBuilder;

    /**
     * The directory where plans will be placed (using a maven repository layout).
     * @since 2.0.0
     */
    @Parameter(alias = "repositoryDirectory",
               defaultValue = "${project.build.directory}/repository")
    private File repository;

    /**
     * Allow timestamp in SNAPSHOT version if set to true
     * @since 2.0.0
     */
    @Parameter(defaultValue = "false")
    private boolean allowTimestamp = false;

    public void execute() throws MojoExecutionException, MojoFailureException {

        // Dependencies have already been resolved
        // We just need to have them as a graph (to know relationship to the main artifact)
        try {

            // Creates the graph
            DependencyNode root = graphBuilder.buildDependencyGraph(project, null);

            // Filter
            PlanAndChildrenCollector collector = new PlanAndChildrenCollector();
            root.accept(collector);

            // Process filtered dependencies
            for (DependencyNode node : collector.getNodes()) {
                // Process the node's Artifact
                process(node.getArtifact());
            }
        } catch (DependencyGraphBuilderException e) {
            throw new MojoExecutionException("Cannot build dependencies graph of project " + project.getArtifactId(), e);
        } catch (NullPointerException npe) {
            getLog().info(npe);
            throw new MojoFailureException(npe.getMessage());
        }
    }

    private void process(Artifact artifact) throws MojoFailureException {
        getLog().debug("Selected artifact: " + artifact);

        // TODO Maybe we should create a real repository and then install artifacts inside ...

        // Copy the artifact into the defined destination
        // Keep the maven2 layout style in the destination repository
        String path = layout.pathOf(artifact);
        File source = artifact.getFile();
        if (source == null) {
            // source may be null when artifact.isResolved() is false.
            source =  new File(DefaultRepositoryRequest.getRepositoryRequest(mavenSession, project).getLocalRepository().getBasedir()+File.separatorChar+path);
        }

        String targetPath = path;
        if (!allowTimestamp) {
            targetPath = path.replace(artifact.getVersion(), artifact.getBaseVersion());
        }
        File destination = new File(repository, targetPath);

        getLog().debug("Copy artifact: " + source + " to " + destination);
        try {
            FileUtils.copyFileIfModified(source, destination);
        } catch (IOException e) {
            throw new MojoFailureException(e.getMessage());
        }
    }

    /**
     * @return true if timestamp are allowed in SNAPSHOT version
     */
    public boolean isAllowTimestamp() {
        return this.allowTimestamp;
    }

    /**
     * @param allowTimestamp Allow timestamp in SNAPSHOT version if set to true
     */
    public void setAllowTimestamp(final boolean allowTimestamp) {
        this.allowTimestamp = allowTimestamp;
    }
}
