/**
 * Copyright 2011-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.deploymentplan.visitor;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.shared.dependency.graph.DependencyNode;
import org.apache.maven.shared.dependency.graph.traversal.DependencyNodeVisitor;

/**
 * A dependency node visitor that collects visited nodes for further processing.
 * It only accept node related to artifact of type 'deployment-plan' and their direct children.
 */
public class PlanAndChildrenCollector implements DependencyNodeVisitor {

    /**
     * The collected list of nodes.
     */
    private final List<DependencyNode> nodes;

    /**
     * Creates a dependency node visitor that collects visited nodes for further processing.
     */
    public PlanAndChildrenCollector() {
        nodes = new ArrayList<DependencyNode>();
    }

    /**
     * {@inheritDoc}
     */
    public boolean visit(DependencyNode node) {
        // collect node if parent is a deployment-plan ...
        if (node.getParent() != null) {
            DependencyNode parent = node.getParent();
            if (isDeploymentPlan(parent.getArtifact())) {
                nodes.add(node);
                return true;
            }
        }

        // ... or if it is a plan by itself
        if (isDeploymentPlan(node.getArtifact())) {
            nodes.add(node);
            return true;
        }

        return true;
    }

    private boolean isDeploymentPlan(Artifact artifact) {
        return "deployment-plan".equals(artifact.getType());
    }

    /**
     * {@inheritDoc}
     */
    public boolean endVisit(DependencyNode node) {
        return true;
    }

    /**
     * Gets the collected dependency nodes.
     * 
     * @return the collected dependency nodes
     */
    public List<DependencyNode> getNodes() {
        return Collections.unmodifiableList(nodes);
    }
}
