/**
 * Copyright 2011-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.deploymentplan;

import java.io.File;

import org.apache.maven.plugin.testing.AbstractMojoTestCase;
import org.codehaus.plexus.util.FileUtils;

/**
 * The VersionsMojoTestCase is ...
 *
 * @author Guillaume Sauthier
 */
public class GeneratePlanMojoTestCase extends AbstractMojoTestCase {


    private File target;

    @Override
    protected void setUp() throws Exception {
        // required for mojo lookups to work
        super.setUp();

        // Safety
        target = new File(new File(getBasedir(), "target"), "generated-plan.xml");
        FileUtils.fileDelete(target.getPath());
        assertFalse("generated-plan.xml file was not removed", target.exists());

    }

    @Override
    protected void tearDown() throws Exception {
        FileUtils.fileDelete(target.getPath());
        super.tearDown();
    }

    public void testMojoFileCreation() throws Exception {

        GeneratePlanMojo mojo = lookupGeneratePlanMojo("test-generate-mojo.pom");
        mojo.setTarget(target);
        mojo.execute();

        assertTrue("Deployment plan was not generated", target.exists());
    }

    public void testDeploymentHasReloadableAttribute() throws Exception {

        GeneratePlanMojo mojo = lookupGeneratePlanMojo("test-reloadable-attribute.pom");
        mojo.setTarget(target);
        mojo.execute();

        assertTrue("Deployment plan was not generated", target.exists());
        assertTrue(FileUtils.fileRead(target).contains("reloadable=\"true\""));
    }

    public void testDeploymentHasReferenceAttribute() throws Exception {

        GeneratePlanMojo mojo = lookupGeneratePlanMojo("test-reference-attribute.pom");
        mojo.setTarget(target);
        mojo.execute();

        assertTrue("Deployment plan was not generated", target.exists());
        assertTrue(FileUtils.fileRead(target).contains("reference=\"true\""));
    }

    public void testDeploymentHasStartAttribute() throws Exception {

        GeneratePlanMojo mojo = lookupGeneratePlanMojo("test-start-attribute.pom");
        mojo.setTarget(target);
        mojo.execute();

        assertTrue("Deployment plan was not generated", target.exists());
        assertTrue(FileUtils.fileRead(target).contains("start=\"false\""));
    }

    public void testDeploymentHasTransientAttribute() throws Exception {

        GeneratePlanMojo mojo = lookupGeneratePlanMojo("test-transient-attribute.pom");
        mojo.setTarget(target);
        mojo.execute();

        assertTrue("Deployment plan was not generated", target.exists());
        assertTrue(FileUtils.fileRead(target).contains("transient=\"true\""));
    }

    public void testDeploymentHasStartLevelAttribute() throws Exception {

        GeneratePlanMojo mojo = lookupGeneratePlanMojo("test-startlevel-attribute.pom");
        mojo.setTarget(target);
        mojo.execute();

        assertTrue("Deployment plan was not generated", target.exists());
        assertTrue(FileUtils.fileRead(target).contains("startlevel=\"2\""));
    }

    public void testDeploymentPlanHasGlobalConditionAttribute() throws Exception {

        GeneratePlanMojo mojo = lookupGeneratePlanMojo("test-global-condition.pom");
        mojo.setTarget(target);
        mojo.execute();

        assertTrue("Deployment plan was not generated", target.exists());
        assertTrue(FileUtils.fileRead(target).contains("condition=\"bundle(0).installed\""));
    }

    public void testDeploymentHasConditionAttribute() throws Exception {

        GeneratePlanMojo mojo = lookupGeneratePlanMojo("test-deployment-enrichment.pom");
        mojo.setTarget(target);
        mojo.execute();

        assertTrue("Deployment plan was not generated", target.exists());
        assertTrue(FileUtils.fileRead(target).contains("condition=\"bundle(1).installed\""));
    }

    private GeneratePlanMojo lookupGeneratePlanMojo(String name) throws Exception {
        File testPom = new File(getBasedir(),
                                "src/test/resources/" + name);
        GeneratePlanMojo mojo = (GeneratePlanMojo) lookupMojo("generate", testPom);
        assertNotNull(mojo);
        return mojo;
    }
}
