/**
 * Copyright 2011-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.deploymentplan;

import java.io.File;
import java.util.Collections;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.DefaultArtifact;
import org.apache.maven.artifact.resolver.ArtifactResolutionRequest;
import org.apache.maven.artifact.resolver.ArtifactResolutionResult;
import org.apache.maven.plugin.testing.AbstractMojoTestCase;
import org.apache.maven.repository.RepositorySystem;
import org.codehaus.plexus.util.FileUtils;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.ow2.util.maven.deploymentplan.mock.TestableMavenProject;
import org.ow2.util.maven.deploymentplan.util.MavenUtils;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.when;

/**
 * The LegacyRepositoryProvisioningMojoTestCase is ...
 *
 * @author Guillaume Sauthier
 */
public class LegacyRepositoryProvisioningMojoTestCase extends AbstractMojoTestCase {

    private File repository;

    @Mock
    private RepositorySystem repositorySystem;

    private DefaultArtifact plan3Artifact = new DefaultArtifact("plan-tests", "plan-3", "1.0", null, "deployment-plan", null, MavenUtils.PLAN_HANDLER);
    private DefaultArtifact plan2Artifact = new DefaultArtifact("plan-tests", "plan-2", "1.0", null, "deployment-plan", null, MavenUtils.PLAN_HANDLER);
    private DefaultArtifact plan1Artifact = new DefaultArtifact("plan-tests", "plan-1", "1.0", null, "deployment-plan", null, MavenUtils.PLAN_HANDLER);
    private DefaultArtifact mavenPluginApiArtifact = new DefaultArtifact("org.apache.maven", "maven-plugin-api", "3.0", null, "jar", null, MavenUtils.JAR_HANDLER);
    private DefaultArtifact mavenSettingsArtifact = new DefaultArtifact("org.apache.maven", "maven-settings", "3.0", null, "jar", null, MavenUtils.JAR_HANDLER);
    private DefaultArtifact mavenProjectArtifact = new DefaultArtifact("org.apache.maven", "maven-project", "2.2.1", null, "jar", null, MavenUtils.JAR_HANDLER);

    @Mock
    private ArtifactResolutionResult plan3Result;
    @Mock
    private ArtifactResolutionResult plan2Result;
    @Mock
    private ArtifactResolutionResult plan1Result;
    @Mock
    private ArtifactResolutionResult mavenPluginApiResult;
    @Mock
    private ArtifactResolutionResult mavenSettingsResult;
    @Mock
    private ArtifactResolutionResult mavenProjectResult;

    @Override
    protected void setUp() throws Exception {
        // required for mojo lookups to work
        super.setUp();
        repository = new File(new File(getBasedir(), "target"), "repository2");
        MockitoAnnotations.initMocks(this);

        plan3Artifact.setFile(getTestFile("src/test/resources/repository/plan-tests/plan-3/1.0/plan-3-1.0.xml"));
        plan2Artifact.setFile(getTestFile("src/test/resources/repository/plan-tests/plan-2/1.0/plan-2-1.0.xml"));
        plan1Artifact.setFile(getTestFile("src/test/resources/repository/plan-tests/plan-1/1.0/plan-1-1.0.xml"));
        mavenPluginApiArtifact.setFile(getTestFile("src/test/resources/repository/org/apache/maven/maven-plugin-api/3.0/maven-plugin-api-3.0.jar"));
        mavenSettingsArtifact.setFile(getTestFile("src/test/resources/repository/org/apache/maven/maven-settings/3.0/maven-settings-3.0.jar"));
        mavenProjectArtifact.setFile(getTestFile("src/test/resources/repository/org/apache/maven/maven-project/2.2.1/maven-project-2.2.1.jar"));
    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
        FileUtils.fileDelete(repository.getPath());
    }

    public void testMojoFileCreation() throws Exception {

        LegacyRepositoryProvisioningMojo mojo = lookupLegacyRepositoryProvisioningMojo("test-legacy-provisioning.pom");
        mojo.execute();

        // Ensure plans + deps have been copied
        File plan1 = new File(repository, nativePath("plan-tests/plan-1/1.0/plan-1-1.0.xml"));
        File plan2 = new File(repository, nativePath("plan-tests/plan-2/1.0/plan-2-1.0.xml"));
        File plan3 = new File(repository, nativePath("plan-tests/plan-3/1.0/plan-3-1.0.xml"));

        File mavenPluginApi = new File(repository, nativePath("org/apache/maven/maven-plugin-api/3.0/maven-plugin-api-3.0.jar"));
        File mavenSettings = new File(repository, nativePath("org/apache/maven/maven-settings/3.0/maven-settings-3.0.jar"));
        File mavenProject = new File(repository, nativePath("org/apache/maven/maven-project/2.2.1/maven-project-2.2.1.jar"));

        assertFileExists(plan1);
        assertFileExists(plan2);
        assertFileExists(plan3);
        assertFileExists(mavenPluginApi);
        assertFileExists(mavenSettings);
        assertFileExists(mavenProject);
    }

    private static String nativePath(String path) {
        return path.replace('/', File.separatorChar);
    }

    private static void assertFileExists(File file) {
        assertTrue("File '" + file + "' do not exists.", file.exists());
    }

    private LegacyRepositoryProvisioningMojo lookupLegacyRepositoryProvisioningMojo(String name) throws Exception {
        File testPom = new File(getBasedir(),
                                "src/test/resources/" + name);

        when(repositorySystem.createArtifact(anyString(), anyString(), anyString(), anyString(), anyString()))
                .thenReturn(plan3Artifact, plan2Artifact);
        when(repositorySystem.createArtifactWithClassifier(anyString(), anyString(), anyString(), anyString(), anyString()))
                .thenReturn(plan1Artifact, mavenPluginApiArtifact, mavenSettingsArtifact, mavenProjectArtifact);
        when(repositorySystem.resolve(any(ArtifactResolutionRequest.class)))
                .thenReturn(plan3Result, plan1Result, mavenPluginApiResult, mavenSettingsResult, plan2Result, mavenProjectResult);

        linkResult(plan3Result, plan3Artifact);
        linkResult(plan2Result, plan2Artifact);
        linkResult(plan1Result, plan1Artifact);
        linkResult(mavenPluginApiResult, mavenPluginApiArtifact);
        linkResult(mavenSettingsResult, mavenSettingsArtifact);
        linkResult(mavenProjectResult, mavenProjectArtifact);

        getContainer().addComponent(repositorySystem, RepositorySystem.class, null);

        LegacyRepositoryProvisioningMojo mojo = (LegacyRepositoryProvisioningMojo) lookupConfiguredMojo(
                new TestableMavenProject(testPom),
                "legacy-provision-repository"
        );
        assertNotNull(mojo);
        return mojo;
    }

    private void linkResult(ArtifactResolutionResult result, Artifact artifact) {
        when(result.isSuccess()).thenReturn(true);
        when(result.getArtifacts()).thenReturn(Collections.singleton(artifact));
    }


}
