/**
 * Copyright 2011-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.deploymentplan;

import java.io.File;

import org.apache.maven.artifact.DefaultArtifact;
import org.apache.maven.artifact.resolver.filter.ArtifactFilter;
import org.apache.maven.plugin.testing.AbstractMojoTestCase;
import org.apache.maven.project.MavenProject;
import org.apache.maven.shared.dependency.graph.DependencyGraphBuilder;
import org.apache.maven.shared.dependency.graph.DependencyNode;
import org.codehaus.plexus.util.FileUtils;
import org.ow2.util.maven.deploymentplan.mock.TestableMavenProject;
import org.ow2.util.maven.deploymentplan.util.MavenUtils;

import static java.util.Arrays.asList;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.isNull;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * The RepositoryProvisioningMojoTestCase is ...
 *
 * @author Guillaume Sauthier
 */
public class RepositoryProvisioningMojoTestCase extends AbstractMojoTestCase {

    private File repository;

    private DefaultArtifact mainArtifact = new DefaultArtifact("plan-tests", "main", "1.0", null, "jar", null, MavenUtils.PLAN_HANDLER);
    private DefaultArtifact plan3Artifact = new DefaultArtifact("plan-tests", "plan-3", "1.0", null, "deployment-plan", null, MavenUtils.PLAN_HANDLER);
    private DefaultArtifact plan2Artifact = new DefaultArtifact("plan-tests", "plan-2", "1.0", null, "deployment-plan", null, MavenUtils.PLAN_HANDLER);
    private DefaultArtifact plan1Artifact = new DefaultArtifact("plan-tests", "plan-1", "1.0", null, "deployment-plan", null, MavenUtils.PLAN_HANDLER);
    private DefaultArtifact mavenPluginApiArtifact = new DefaultArtifact("org.apache.maven", "maven-plugin-api", "3.0", null, "jar", null, MavenUtils.JAR_HANDLER);
    private DefaultArtifact mavenSettingsArtifact = new DefaultArtifact("org.apache.maven", "maven-settings", "3.0", null, "jar", null, MavenUtils.JAR_HANDLER);
    private DefaultArtifact mavenProjectArtifact = new DefaultArtifact("org.apache.maven", "maven-project", "2.2.1", null, "jar", null, MavenUtils.JAR_HANDLER);

    @Override
    protected void setUp() throws Exception {
        // required for mojo lookups to work
        super.setUp();
        repository = new File(new File(getBasedir(), "target"), "repository");

        plan3Artifact.setFile(getTestFile("src/test/resources/repository/plan-tests/plan-3/1.0/plan-3-1.0.xml"));
        plan2Artifact.setFile(getTestFile("src/test/resources/repository/plan-tests/plan-2/1.0/plan-2-1.0.xml"));
        plan1Artifact.setFile(getTestFile("src/test/resources/repository/plan-tests/plan-1/1.0/plan-1-1.0.xml"));
        mavenPluginApiArtifact.setFile(getTestFile("src/test/resources/repository/org/apache/maven/maven-plugin-api/3.0/maven-plugin-api-3.0.jar"));
        mavenSettingsArtifact.setFile(getTestFile("src/test/resources/repository/org/apache/maven/maven-settings/3.0/maven-settings-3.0.jar"));
        mavenProjectArtifact.setFile(getTestFile("src/test/resources/repository/org/apache/maven/maven-project/2.2.1/maven-project-2.2.1.jar"));

    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
        FileUtils.fileDelete(repository.getPath());
    }

    public void testMojoFileCreation() throws Exception {

        RepositoryProvisioningMojo mojo = lookupRepositoryProvisioningMojo("test-provisioning.pom");
        mojo.execute();

        // Ensure plans + deps have been copied
        File plan1 = new File(repository, nativePath("plan-tests/plan-1/1.0/plan-1-1.0.xml"));
        File plan2 = new File(repository, nativePath("plan-tests/plan-2/1.0/plan-2-1.0.xml"));
        File plan3 = new File(repository, nativePath("plan-tests/plan-3/1.0/plan-3-1.0.xml"));

        File mavenPluginApi = new File(repository, nativePath("org/apache/maven/maven-plugin-api/3.0/maven-plugin-api-3.0.jar"));
        File mavenSettings = new File(repository, nativePath("org/apache/maven/maven-settings/3.0/maven-settings-3.0.jar"));
        File mavenProject = new File(repository, nativePath("org/apache/maven/maven-project/2.2.1/maven-project-2.2.1.jar"));

        assertFileExists(plan1);
        assertFileExists(plan2);
        assertFileExists(plan3);
        assertFileExists(mavenPluginApi);
        assertFileExists(mavenSettings);
        assertFileExists(mavenProject);
    }

    private static String nativePath(String path) {
        return path.replace('/', File.separatorChar);
    }

    private static void assertFileExists(File file) {
        assertTrue("File '" + file + "' do not exists.", file.exists());
    }

    private RepositoryProvisioningMojo lookupRepositoryProvisioningMojo(String name) throws Exception {
        File testPom = new File(getBasedir(),
                                "src/test/resources/" + name);
        MavenProject project = new TestableMavenProject(testPom);

        DependencyGraphBuilder builder = mock(DependencyGraphBuilder.class);
        when(builder.buildDependencyGraph(any(MavenProject.class), (ArtifactFilter) isNull()))
                .thenReturn(buildGraph());

        getContainer().addComponent(builder, DependencyGraphBuilder.class, "maven3");
        RepositoryProvisioningMojo mojo = (RepositoryProvisioningMojo) lookupConfiguredMojo(project, "provision-repository");
        assertNotNull(mojo);
        return mojo;
    }

    private DependencyNode buildGraph() {
        // Tree structure returned:
        // plan-tests:main:jar:1.0
        // -> plan-tests:plan-3:deployment-plan:1.0
        //    -> plan-tests:plan-1:deployment-plan:1.0
        //       -> org.apache.maven:maven-plugin-api:jar:3.0
        //       -> org.apache.maven:maven-settings:jar:3.0
        // -> plan-tests:plan-2:deployment-plan:1.0
        //    -> org.apache.maven:maven-project:jar:2.2.1

        DependencyNode mainNode = MavenUtils.dependencyNode(null, mainArtifact);
        DependencyNode plan3Node = MavenUtils.dependencyNode(mainNode, plan3Artifact);
        DependencyNode plan2Node = MavenUtils.dependencyNode(mainNode, plan2Artifact);
        DependencyNode plan1Node = MavenUtils.dependencyNode(plan3Node, plan1Artifact);
        DependencyNode mavenPluginApiNode = MavenUtils.dependencyNode(plan1Node, mavenPluginApiArtifact);
        DependencyNode mavenSettingsNode = MavenUtils.dependencyNode(plan1Node, mavenSettingsArtifact);
        DependencyNode mavenProjectNode = MavenUtils.dependencyNode(plan2Node, mavenProjectArtifact);

        mainNode.getChildren().addAll(asList(plan3Node, plan2Node));
        plan3Node.getChildren().addAll(asList(plan1Node));
        plan1Node.getChildren().addAll(asList(mavenPluginApiNode, mavenSettingsNode));
        plan2Node.getChildren().addAll(asList(mavenProjectNode));

        return mainNode;
    }

}
